Unit FrmMain;

//  ////////////////////////////////////////////////////////////////////////////
//
//  Copyright (C) 2003 Jan Punter
//
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
//
//  ////////////////////////////////////////////////////////////////////////////
//
//    lupd : 2003-10-13
//
//  ////////////////////////////////////////////////////////////////////////////


Interface

Uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, StdCtrls, Buttons, ExtCtrls, Inifiles,

  Globals, LexLib, pdlstructs, patchstructs, patchparser;

Type

  TFormMain = Class(TForm)
    PanelTop: TPanel;
    Label1: TLabel;
    EditTestFileName: TEdit;
    BitBtnBrowseTest: TBitBtn;
    BitBtnProcessTest: TBitBtn;
    BitBtnExit: TBitBtn;
    CheckBoxTestDebug: TCheckBox;
    PanelMain: TPanel;
    MemoResults: TMemo;
    OpenDialogTest: TOpenDialog;
    BitBtn1: TBitBtn;
    procedure FormShow(Sender: TObject);
    procedure FormClose(Sender: TObject; var Action: TCloseAction);
    procedure BitBtnExitClick(Sender: TObject);
    procedure BitBtnBrowseTestClick(Sender: TObject);
    procedure BitBtnProcessTestClick(Sender: TObject);
    procedure CheckBoxTestDebugClick(Sender: TObject);
    procedure BitBtn1Click(Sender: TObject);
  Private
    FTestFileName : String;
    FTestDebug    : Boolean;
    Fpatch        : TPatch;
  Private
    Procedure   SetTestFileName( Const aValue: String);
    Procedure   SetTestDebug( avalue: Boolean);
    Procedure   DoDebug( aSender: TObject; Const aMsg: String);
  Private
    Procedure   LoadIni;
    Procedure   SaveIni;
  Private
    Procedure   BrowseForTestFile;
    Procedure   ProcessTestFile;
    Procedure   DumpPatch;
  Private
    Property    TestFileName: String  Read FTestFileName Write SetTestFileName;
    Property    TestDebug   : Boolean Read FTestDebug    Write SetTestDebug;
  End;

Var
  FormMain: TFormMain;

Implementation

{$R *.dfm}

// User area

Type

  TTestTracer = Class( TTracer)
  Private
    FStrings : TStrings;
  Public
    Constructor Create( aStrings: TStrings);
    Procedure   Trace( Const aMsg: String); Override;
  End;

    Constructor TTestTracer.Create( aStrings: TStrings);
    Begin
      Inherited Create;
      FStrings := aStrings;
      If Assigned( FStrings)
      Then FStrings.Clear;
    End;

    Procedure   TTestTracer.Trace( Const aMsg: String); // Override;
    Begin
      If Assigned( FStrings)
      Then FStrings.Add( aMsg);
    End;

//  Private

    Procedure   TFormMain.SetTestFileName( Const aValue: String);
    Begin
      If aValue <> FTestFileName
      Then Begin
        FTestFileName := aValue;
        EditTestFileName.Text := FTestFileName;
      End;
    End;

    Procedure   TFormMain.SetTestDebug( avalue: Boolean);
    Begin
      If avalue <> FTestDebug
      Then Begin
        FTestDebug := aValue;
        CheckBoxTestDebug.Checked := FTestDebug;
      End;
    End;

    Procedure   TFormMain.DoDebug( aSender: TObject; Const aMsg: String);
    Begin
      MemoResults.Lines.Add( aMsg);
    End;

//  Private

Const

  _d = 'Defaults';

    Procedure   TFormMain.LoadIni;
    Begin
      With TIniFile.Create( IniFileName) Do
        Try
          TestFileName := ReadString( _d, 'TestFileName', TestFileName);
          TestDebug    := ReadBool  ( _d, 'TestDebug'   , TestDebug    );
        Finally
          Free;
        End;
    End;

    Procedure   TFormMain.SaveIni;
    Begin
      With TIniFile.Create( IniFileName) Do
        Try
          WriteString( _d, 'TestFileName', TestFileName);
          WriteBool  ( _d, 'TestDebug'   , TestDebug    );
        Finally
          Free;
        End;
    End;

//  Private

    Procedure   TFormMain.BrowseForTestFile;
    Begin
      With OpenDialogTest Do
      Begin
        If TestFileName <> ''
        Then FileName := TestFileName;
        If Execute
        Then TestFileName := FileName;
      End;
    End;

    Procedure   TFormMain.ProcessTestFile;
    Var
      i : Integer;
    Begin
      MemoResults.Clear;
      If Assigned( FPatch)
      Then FreeAndNil( FPatch);
      FPatch := TPatch.Create;
      With FPatch Do
      Begin
        OnPatchError := DoDebug;
        LoadPatch( TestFileName, TestDebug);
        If TestDebug
        Then
          With yyOutput Do
            For i := 0 To Count - 1 Do
              MemoResults.Lines.Add( Strings[ i]);
      End;
    End;

    Procedure   TFormMain.DumpPatch;
    Begin
      If Assigned( FPatch)
      Then Begin
        ForceDirectories( ApplicationPath + 'output');
        FPatch.WritePatch( ApplicationPath + 'output\dump.pch');
      End;
    End;

// Delpi area

procedure TFormMain.FormShow(Sender: TObject);
begin
  LoadIni;
end;

procedure TFormMain.FormClose(Sender: TObject; var Action: TCloseAction);
begin
  SaveIni;
  If Assigned( FPatch)
  Then FreeAndNil( FPatch);
end;

procedure TFormMain.BitBtnExitClick(Sender: TObject);
begin
  Close;
end;

procedure TFormMain.BitBtnBrowseTestClick(Sender: TObject);
begin
  BrowseForTestFile;
end;

procedure TFormMain.BitBtnProcessTestClick(Sender: TObject);
begin
  ProcessTestFile;
end;

procedure TFormMain.CheckBoxTestDebugClick(Sender: TObject);
begin
  TestDebug := CheckBoxTestDebug.Checked;
end;

procedure TFormMain.BitBtn1Click(Sender: TObject);
begin
  DumpPatch;
end;

End.
