%{

// /////////////////////////////////////////////////////////////////////////////
//
//  Nord Modular patch file format 3.03 parser
//  Copyright (C) 2003 Marcus Andersson
//
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
//
//  ////////////////////////////////////////////////////////////////////////////
//
//  Modified to be used in Delphi 7 by Jan Punter.
//
//  Modifications CopyRight (c) 2003 Jan Punter
//
//  Distributed under the same conditions as the original code by Narcus
//  Andersson.
//
//  This should be compileable for the tp yacc vession 3.0 as taken from
//  the free pascal distribution. Modified versions of lexlib.pas, yacclib.pas,
//  yyylex.cod and yyparse.cod were used. These should be present in this
//  distribution.
//
//  Notes :
//
//    Take care with quoting in the comments above (and below), an uneven number
//    of quotes in this section will give compile errors from pyacc.
//
//    The lexical analyzer created from pdllex.l is included in the code
//    generated from this file, and it _must_ be included.
//
//    lupd : 2003-10-16
//
//  ////////////////////////////////////////////////////////////////////////////



Unit patchparser;

Interface

{$WARN UNSAFE_TYPE OFF}

Uses

  SysUtils, Classes, patchstructs;


Type

  YYSType = String;

Var

  GPatch          : TPatch;
  GSection        : TModuleSection;
  GModule         : TModule;
  GMorph          : TMorph;
  GMorphMap       : TMorphMap;
  GKnobMap        : TKnobMap;
  GCtrlMap        : TCtrlMap;
  GParameterIndex : Integer;
  GCustomIndex    : Integer;
  GStrings        : TStringList;


  Procedure InitPatch( aPatch: TPatch);

%}

%token PARAM
%token NMVERSION
%token HEADER
%token HEADER_END
%token MODULE_DUMP
%token MODULE_DUMP_END
%token CURRENT_NOTE_DUMP
%token CURRENT_NOTE_DUMP_END
%token CABLE_DUMP
%token CABLE_DUMP_END
%token PARAMETER_DUMP
%token PARAMETER_DUMP_END
%token CUSTOM_DUMP
%token CUSTOM_DUMP_END
%token MORPH_MAP_DUMP
%token MORPH_MAP_DUMP_END
%token KEYBOARD_ASSIGNMENT
%token KEYBOARD_ASSIGNMENT_END
%token KNOB_MAP_DUMP
%token KNOB_MAP_DUMP_END
%token CTRL_MAP_DUMP
%token CTRL_MAP_DUMP_END
%token NAME_DUMP
%token NAME_DUMP_END
%token NOTES

%%

patch
  : patch patch_part
  |
  ;

patch_part
  : header
  | module_dump
  | module_dump_300
  | current_note_dump
  | cable_dump
  | cable_dump_300
  | parameter_dump
  | custom_dump
  | morph_map_dump
  | keyboard_assignment
  | knob_map_dump
  | ctrl_map_dump
  | name_dump
  | notes
  | '\n'
  ;

header
  : HEADER '\n'
    NMVERSION '\n'
    PARAM PARAM PARAM PARAM PARAM
    PARAM PARAM PARAM PARAM PARAM
    PARAM PARAM PARAM PARAM PARAM
    PARAM PARAM PARAM PARAM PARAM
    PARAM PARAM PARAM '\n'
    HEADER_END
    {
      // header : HEADER '\n' NMVERSION '\n' PARAM (13,13) '\n' HEADER_END _

      With GPatch Do
      Begin
        KeyboardRange[ rlMin] := StrToInt( Trim(  $5));
        KeyboardRange[ rlMax] := StrToInt( Trim(  $6));
        VelocityRange[ rlMin] := StrToInt( Trim(  $7));
        VelocityRange[ rlMax] := StrToInt( Trim(  $8));
        BendRange             := StrToInt( Trim(  $9));
        PortamentoTime        := StrToInt( Trim( $10));
        Portamento            := TPortamento( StrToInt( Trim( $11)));
        RequestedVoices       := StrToInt( Trim( $12));
        SectionSeparationPos  := StrToInt( Trim( $13));
        OctaveShift           := StrToInt( Trim( $14));
        ModuleSection[ msPoly  ].VoiceRetrigger := TVoiceRetrigger( StrToInt( Trim( $15)));
        ModuleSection[ msCommon].VoiceRetrigger := TVoiceRetrigger( StrToInt( Trim( $16)));
        CableVisibility[ coRED   ] := Boolean( StrToInt( Trim( $21)));
        CableVisibility[ coBLUE  ] := Boolean( StrToInt( Trim( $22)));
        CableVisibility[ coYELLOW] := Boolean( StrToInt( Trim( $23)));
        CableVisibility[ coGRAY  ] := Boolean( StrToInt( Trim( $24)));
        CableVisibility[ coGREEN ] := Boolean( StrToInt( Trim( $25)));
        CableVisibility[ coPURPLE] := Boolean( StrToInt( Trim( $26)));
        CableVisibility[ coWHITE ] := Boolean( StrToInt( Trim( $27)));
      End;
    }
  ;

module_dump
  : MODULE_DUMP '\n'
    PARAM '\n'
    {
      // module_dump : MODULE_DUMP '\n' PARAM '\n' _ module_list MODULE_DUMP_END

      GSection := GPatch.ModuleSection[ TTModuleSection( StrToInt( Trim( $3)))];
    }
    module_list
    MODULE_DUMP_END
  ;

module_dump_300
  : MODULE_DUMP '\n'
    PARAM
    {
      // module_dump_300 : MODULE_DUMP '\n' PARAM _ module_list MODULE_DUMP_END

      GSection := GPatch.ModuleSection[ TTModuleSection( StrToInt( Trim( $3)))];
    }
    module_list
    MODULE_DUMP_END
  ;

module_list
  : module_list PARAM PARAM PARAM PARAM '\n'
    {
      // module_list : module_list PARAM (4,4) _

      GModule := GSection.NewModule( StrToInt( Trim( $3)), StrToInt( Trim( $2)));
      With GModule Do
      Begin
        XPosition := StrToInt( Trim( $4));
        YPosition := StrToInt( Trim( $5));
      End;
    }
  | '\n'
  |
  ;

current_note_dump
  : CURRENT_NOTE_DUMP '\n'
    note_list '\n'
    CURRENT_NOTE_DUMP_END
  ;

note_list
  : note_list PARAM PARAM PARAM
    {
      // note_list : note_list PARAM (3,3) _

      GPatch.NewMidiNote( StrToInt( Trim( $2)), StrToInt( Trim( $3)), StrToInt( Trim( $4)));
    }
  |
  ;

cable_dump
  : CABLE_DUMP '\n'
    PARAM '\n'
    {
      // cable_dump : CABLE_DUMP '\n' PARAM '\n' _ cable_list CABLE_DUMP_END
      
      GSection := GPatch.ModuleSection[ TTModuleSection( StrToInt( Trim( $3)))];
    }
    cable_list
    CABLE_DUMP_END
  ;

cable_dump_300
  : CABLE_DUMP '\n'
    PARAM
    {
      // cable_dump_300 : CABLE_DUMP '\n' PARAM _ cable_list CABLE_DUMP_END

      GSection := GPatch.ModuleSection[ TTModuleSection( StrToInt( Trim( $3)))];
    }
    cable_list
    CABLE_DUMP_END
  ;

cable_list
  : cable_list PARAM PARAM PARAM PARAM PARAM PARAM PARAM '\n'
    {
      // cable_list : cable_list PARAM (7,7) '\n' _

      GSection.
        NewCable(
          TTColor(     StrToInt( Trim( $2))),
                       StrToInt( Trim( $3)) ,
                       StrToInt( Trim( $4)) ,
          TTConnector( StrToInt( Trim( $5))),
                       StrToInt( Trim( $6)) ,
                       StrToInt( Trim( $7)) ,
          TTConnector( StrToInt( Trim( $8)))
        );
    }
  | '\n'
  |
  ;

parameter_dump
  : PARAMETER_DUMP '\n'
    PARAM '\n'
    {
      // @@@@1
      // parameter_dump : PARAMETER_DUMP '\n' PARAM '\n' _ module_parameter_list PARAMETER_DUMP_END
      
      GSection := GPatch.ModuleSection[ TTModuleSection( StrToInt( Trim( $3)))];
    }
    module_parameter_list
    PARAMETER_DUMP_END
  ;

module_parameter_list
  : module_parameter_list PARAM PARAM PARAM 
    {
      // module_parameter_list : module_parameter_list PARAM (3,3) _ parameter_list '\n'

      GModule := GSection.Module[ StrToInt( Trim( $2))];
      GParameterIndex := 0;
    }
    parameter_list '\n'
  | '\n'
  |
  ;

parameter_list
  : parameter_list PARAM
    {
      // parameter_list : parameter_list PARAM _

      GModule.Parameter[ GParameterIndex] := StrToInt( Trim( $2));
      Inc( GParameterIndex);
    }
  |
  ;

custom_dump
  : CUSTOM_DUMP '\n'
    PARAM
    {
      // custom_dump CUSTOM_DUMP '\n' PARAM _ module_custom_list CUSTOM_DUMP_END

      GSection := GPatch.ModuleSection[ TTModuleSection( StrToInt( Trim( $3)))];
    }
    module_custom_list
    CUSTOM_DUMP_END
  ;

module_custom_list
  : module_custom_list PARAM PARAM
    {
      // module_custom_list : module_custom_list PARAM (2,2) _ custom_list '\n'

      GModule := GSection.Module[ StrToInt( Trim( $2))];
      GCustomIndex := 0;
    }
    custom_list '\n'
  | '\n'
  |
  ;

custom_list
  : custom_list PARAM
    {
      // custom_list : custom_list PARAM _

      GModule.Custom[ GCustomIndex] := StrToInt( Trim( $2));
      Inc( GCustomIndex);
    }
  |
  ;

morph_map_dump
  : MORPH_MAP_DUMP '\n'
    PARAM PARAM PARAM PARAM '\n'
    {
      // morph_map_dump : MORPH_MAP_DUMP '\n' PARAM (4,4) '\n' _ morph_map_list '\n' MORPH_MAP_DUMP_END

      With GPatch Do
      Begin
        Morph[ mtMorph1].Value := StrToInt( Trim( $3));
        Morph[ mtMorph2].Value := StrToInt( Trim( $4));
        Morph[ mtMorph3].Value := StrToInt( Trim( $5));
        Morph[ mtMorph4].Value := StrToInt( Trim( $6));
      End;
    }
    morph_map_list '\n'
    MORPH_MAP_DUMP_END
  ;

morph_map_list
  : morph_map_list PARAM PARAM PARAM PARAM PARAM
    {
      // morph_map_list : morph_map_list PARAM (5,5) _

      GSection  := GPatch.ModuleSection[ TTModuleSection( StrToInt( Trim( $2)))];
      GModule   := GSection.Module[ StrToInt( Trim( $3))];
      GMorph    := GPatch.Morph[ TTMorph( StrToInt( Trim( $5)))];
      GMorphMap := GMorph.NewMorphMap( TTModuleSection( StrToInt( Trim( $2))), GModule, StrToInt( Trim( $4)));
      GMorphMap.Range := StrToInt( Trim( $6));
    }
  |
  ;

keyboard_assignment
  : KEYBOARD_ASSIGNMENT '\n'
    PARAM PARAM PARAM PARAM '\n'
    {
      // keyboard_assignment : KEYBOARD_ASSIGNMENT '\n' PARAM (4,4) '\n' _ KEYBOARD_ASSIGNMENT_END

      With Gpatch Do
      Begin
        Morph[ mtMorph1].KeyboardAssignment := TKeyboardAssignment( StrToInt( Trim( $3)));
        Morph[ mtMorph2].KeyboardAssignment := TKeyboardAssignment( StrToInt( Trim( $4)));
        Morph[ mtMorph3].KeyboardAssignment := TKeyboardAssignment( StrToInt( Trim( $5)));
        Morph[ mtMorph4].KeyboardAssignment := TKeyboardAssignment( StrToInt( Trim( $6)));
      End;
    }
    KEYBOARD_ASSIGNMENT_END
  ;

knob_map_dump
  : KNOB_MAP_DUMP '\n'
    knob_map_list
    KNOB_MAP_DUMP_END
  ;

knob_map_list
  : knob_map_list PARAM PARAM PARAM PARAM '\n'
    {
      // knob_map_list : knob_list_map PARAM (4,4) '\n' _

      If  StrToInt( Trim( $2)) = Ord( msMorph)
      Then GModule := Nil
      Else Begin
        GSection := GPatch.ModuleSection[ TTmoduleSection( StrToInt( Trim( $2)))];
        GModule  := GSection.Module[ StrToInt( Trim( $3))];
      End;
      GKnobMap := GPatch.NewKnobMap( TTModuleSection( StrToInt( Trim( $2))), GModule, StrToInt( Trim( $4)));
      GKnobMap.Knob := StrToInt( Trim( $5));
    }
  | '\n'
  |
  ;

ctrl_map_dump
  : CTRL_MAP_DUMP '\n'
    ctrl_map_list
    CTRL_MAP_DUMP_END
  ;

ctrl_map_list
  : ctrl_map_list PARAM PARAM PARAM PARAM '\n'
    {
      // ctrl_map_list : ctrl_map_list PARAM (4,4) '\n' _

      If  StrToInt( Trim( $2)) = Ord( msMorph)
      Then GModule := Nil
      Else Begin
        GSection := GPatch.ModuleSection[ TTmoduleSection( StrToInt( Trim( $2)))];
        GModule  := GSection.Module[ StrToInt( Trim( $3))];
      End;
      GCtrlMap := GPatch.NewCtrlMap( TTModuleSection( StrToInt( Trim( $2))), GModule, StrToInt( Trim( $4)));
      GCtrlMap.CC := StrToInt( Trim( $5));
    }
  | '\n'
  |
  ;

name_dump
  : NAME_DUMP '\n'
    PARAM '\n'
    {
      // name_dump : NAMEDUMP '\n' PARAM '\n' _ name_list NAME_DUMP_END

      GSection := GPatch.ModuleSection[ TTmoduleSection( StrToInt( Trim( $3)))];
    }
    name_list
    NAME_DUMP_END
  ;

name_list
  : name_list PARAM name '\n'
    {
      // name_list : name_list PARAM name '\n' _

      GModule := GSection.Module[ StrToInt( Trim( $2))];
      GModule.Name := $3;
    }
  |
  ;

name
  : name PARAM
    {
      // name : name PARAM _

      $$ := $1 + $2;
    }
  |
    {
      // name : _

      $$ := '';
    }
  ;

notes
  : NOTES
    {
      // notes : NOTES _

      With GStrings Do
      Begin
        SetText( PChar( $1));
        If ( Count > 0) And ( Strings[ 0] = '')
        Then Delete( 0);
      End;
      GPatch.Notes := GStrings;
    }
  ;

%%

  Procedure InitPatch( aPatch: TPatch);
  Begin
    GPatch := aPatch;
  End;

  
// /////////////////////////////////////////////////////////////////////////////
// Include the lexer

{$INCLUDE patchlexer.pas}

// End  of lexer.
// /////////////////////////////////////////////////////////////////////////////


Initialization

  GStrings := TStringList.Create;

Finalization

  If Assigned( GStrings)
  Then FreeAndNil( GStrings);

End.


