
//  ////////////////////////////////////////////////////////////////////////////
//
//  Protocol Definition Language
//  Copyright (C) 2003 Marcus Andersson
//
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
//
//  ////////////////////////////////////////////////////////////////////////////
//
//  Modified to be used in Delphi 7 by Jan Punter.
//
//  Modifications CopyRight (c) 2003 Jan Punter
//
//  Distributed under the same conditions as the original code by Narcus
//  Andersson.
//
//  Notes :
//
//   This is different from how the C versions of Lex and Yacc work.
//   This file is meant to be translated into a pascal file to be included
//   in the code generated from pdlparse.y, no stand alone unit is therefore
//   generated here.
//
//   Some initialization stuff was removed from here as it was already
//   supported by either lexlib.pas, yylex.cod or yyparse.cod.
//
//   This should be compileable for the tp lex vession 3.0 as taken from
//   the free pascal distribution. Modified versions of lexlib.pas, yyylex.cod
//   and yyparse.cod were used. These should be present in this distribution.
//
//    The word END was used as a terminal token, this had to be changed as
//    END is a reserved word in Pascal. I changed it into _END.
//
//    lupd : 2003-10-13
//
//  ////////////////////////////////////////////////////////////////////////////


%{
%}


%%

"0x"[0-9a-f]*
    Begin
      // 0x"[0-9a-f]
      yylval := '$' + Copy( yyText, 3, Length( yyText) - 2);
      yyDebugOutFmt( 'NUMBER %s', [ yylval]);
      return( NUMBER);
    End;

[0-9]*
    Begin
      // [0-9]*
      yylval := yytext;
      yyDebugOutFmt( 'NUMBER %s', [ yylval]);
      return( NUMBER);
    End;

[a-zA-Z][a-zA-Z0-9]*
    Begin
      // [a-zA-Z][a-zA-Z0-9]*
      yylval := yyText;
      yyDebugOutFmt( 'IDENTIFIER %s', [ yylval]);
      return( IDENTIFIER);
    End;

%
    Begin
      // %
      yyDebugOut( 'PAD %');
      return( PAD);
    End;

":="
    Begin
      // :=
      yyDebugOut( 'ASSIGN :=');
      return( ASSIGN);
    End;

"=>"
    Begin
      // =>
      yyDebugOut( 'CHOOSE =>');
      return( CHOOSE);
    End;

\*
    Begin
      // *
      yyDebugOut( 'TIMES *');
      return( TIMES);
    End;

\/
    Begin
      // /
      yyDebugOut( 'STOP /');
      return( STOP);
    End;

=
    Begin
      // =
      yyDebugOut( 'COMPARE =');
      return( COMPARE);
    End;

!
    Begin
      // !
      yyDebugOut( 'NOT_COMPARE !');
      return( NOT_COMPARE);
    End;

\$
    Begin
      // $
      yyDebugOut( 'BIND $');
      return( BIND);
    End;

:
    Begin
      // :
      yyDebugOut( 'SIZE :');
      return( SIZE);
    End;

\;
    Begin
      // ;
      yyDebugOut( '_END ;');
      return( _END);
    End;

\?
    Begin
      // ?
      yyDebugOut( 'OPTIONAL ?');
      return( OPTIONAL);
    End;

#
    Begin
      // #
      yyDebugOut( 'CONDITIONAL #');
      return( CONDITIONAL);
    End;

[ \t]+
    Begin
      // [ \t]+
    End;

\n
    Begin
      // \n
    End;

"//".*$
    Begin
      // // comment to end of line
    End;

"{"((.*)\n)*"}"
    Begin
      // { Comment }
    End;

"/*"((.*)\n)*"*/"
    Begin
      // /* Comment */
    End;

.   // We should handle all input.

%%



