
// Yacc parser template (TP Yacc V3.0), V1.2 6-17-91 AG *)
//
// Modified to be used with Delphi 7 by Jan Punter.



// /////////////////////////////////////////////////////////////////////////////
//
//  Protocol Definition Language
//  Copyright (C) 2003 Marcus Andersson
//
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
//
//  ////////////////////////////////////////////////////////////////////////////
//
//  Modified to be used in Delphi 7 by Jan Punter.
//
//  Modifications CopyRight (c) 2003 Jan Punter
//
//  Distributed under the same conditions as the original code by Narcus
//  Andersson.
//
//  This should be compileable for the tp yacc vession 3.0 as taken from
//  the free pascal distribution. Modified versions of lexlib.pas, yacclib.pas,
//  yyylex.cod and yyparse.cod were used. These should be present in this
//  distribution.
//
//  Notes :
//
//    Take care with quoting in the comments above (and below), an uneven number
//    of quotes in this section will give compile errors from pyacc.
//
//    The lexical analyzer created from pdllex.l is included in the code
//    generated from this file, and it _must_ be included.
//
//    The word END was used as a terminal token, this had to be changed as
//    END is a reserved word in Pascal. I changed it into _END.
//
//    lupd : 2003-10-13
//
//  ////////////////////////////////////////////////////////////////////////////



Unit pdlparser;

Interface

Uses

  SysUtils, Classes, pdlstructs;


Type

  YYSType = String;

Var

  Protocol     : TProtocol;
  PacketParser : TPacketParser;
  Condition    : TCondition;
  IsOptional   : Boolean;      // Optinal clashed with 'OPTIONAL', Pascal is not
                               // case sensitive. Renamed to IsOptional.

const NUMBER = 257;
const IDENTIFIER = 258;
const PAD = 259;
const ASSIGN = 260;
const CHOOSE = 261;
const TIMES = 262;
const STOP = 263;
const OPTIONAL = 264;
const COMPARE = 265;
const NOT_COMPARE = 266;
const BIND = 267;
const SIZE = 268;
const _END = 269;
const CONDITIONAL = 270;

Type

  // Exceptions

  EParser    = Class( Exception);
  ESingleton = Class( EParser  );


  // Callbacks

  TParseError = Procedure( aSender: TObject; Const aMsg: String; Var AbortParsing: Boolean) Of Object;


  // TParser, a wrapper around the Lex / Yacc generated parser.

	TParser = Class
  Private
    FDebug        : Boolean;
    FOnParseError : TParseError;
  Private
    Function    GetToken: Integer;
    Procedure   SetDebug( aValue: Boolean);
    Function    GetErrorCount: Integer;
    Function    GetMessages  : TStrings;
    Function    GetErrorFile : String;
    Function    GetErrorLine : Integer;
    Function    GetErrorCol  : Integer;
	Public
    Constructor Create;
    Destructor  Destroy;                                               Override;
		Function    Parse: Integer;
    Procedure   Clear;
    Procedure   PushFile( Const aFileName: String);
    Procedure   ParseError( Const aMsg: String);
    Procedure   ParseErrorFmt( Const aFormat: String; Const anArgs: Array Of Const);
  Public
    Property    Debug        : Boolean     Read FDebug        Write SetDebug;
    Property    ErrorCount   : Integer     Read GetErrorCount;
    Property    OnParseError : TParseError Read FOnParseError Write FOnParseError;
    Property    Messages     : TStrings    Read GetMessages;
    Property    ErrorFile    : String      Read GetErrorFile;
    Property    ErrorLine    : Integer     Read GetErrorLine;
    Property    ErrorCol     : Integer     Read GetErrorCol;
	End;


Var

  yylval : YYSType;


Implementation

Uses

  lexlib, yacclib;

Var

  ConstructCount : Integer = 0;


{ ========
	TParser = Class
  Private
    FDebug        : Boolean;
    FOnParseError : TParseError;
  Public
    Property    Debug        : Boolean     Read FDebug        Write SetDebug;
    Property    ErrorCount   : Integer     Read GetErrorCount;
    Property    OnParseError : TParseError Read FOnParseError Write FOnParseError;
    Property    ErrorFile    : String      Read GetErrorFile;
    Property    ErrorLine    : Integer     Read GetErrorLine;
    Propert     ErrorCol     : Integer     Read GetErrorCol;
  Private
}

    Procedure   TParser.SetDebug( aValue: Boolean);
    Begin
      FDebug  := aValue;
      yyDebug := FDebug;
    End;

    Function    TParser.GetErrorCount: Integer;
    Begin
      Result := yyErrorCount;
    End;

    Function    TParser.GetMessages: TStrings;
    Begin
      Result := yyOutput;
    End;

    Function    TParser.GetErrorFile: String;
    Begin
      Result := yyFileName;
    End;

    Function    TParser.GetErrorLine: Integer;
    Begin
      Result := yyLineNo;
    End;

    Function    TParser.GetErrorCol: Integer;
    Begin
      Result := yyColNo;
    End;

//  Public

    Constructor TParser.Create;
    Begin
      If ConstructCount > 0
      Then Raise ESingleton.Create( 'Only one parser may be constructed.');
      Inc( ConstructCount);
      Inherited;
      Clear;
    End;

    Destructor  TParser.Destroy; // Override;
    Begin
      If ConstructCount > 0
      Then Dec( ConstructCount);
      Inherited;
    End;

    Procedure   TParser.Clear;
    Begin
      yyInit;
    End;

    Procedure   TParser.PushFile( Const aFileName: String);
    Begin
      If Not FileExists( aFileName)
      Then ParseErrorFmt( 'TParser.PushFile : File not fcound : ''%s''', [ aFileName]);
      yyInclude( aFileName);
    End;

    Procedure   TParser.ParseError( Const aMsg: String);
    Var
      S            : String;
      AbortParsing : Boolean;
    Begin
      S :=
        Format(
          'Error [file=''%s'' line=%d col=%d ] ''%s''',
          [
            ErrorFile,
            ErrorLine,
            ErrorCol,
            aMsg
          ]
        );
      If Assigned( FOnParseError)
      Then Begin
        AbortParsing := True;
        FOnParseError( Self, S, AbortParsing);
        If AbortParsing
        Then Abort;
      End
      Else Raise EParser.Create( S);
    End;

    Procedure   TParser.ParseErrorFmt( Const aFormat: String; Const anArgs: Array Of Const);
    Begin
      ParseError( Format( aFormat, anArgs));
    End;


Function TParser.Parse: Integer;
Var
  yystate : Integer;
  yysp    : Integer;
  yyn     : Integer;
  yys     : array [ 1 .. yymaxdepth] Of Integer;
  yyv     : array [ 1 .. yymaxdepth] Of YYSType;
  yyval   : YYSType;


  Procedure yyaction( yyruleno: Integer);
  // local definitions:


  Begin
    // actions:
    Case yyruleno Of

   1 : begin
         yyval := yyv[yysp-1];
       end;
   2 : begin
       end;
   3 : begin
         
         // packet : IDENTIFIER PAD NUMBER ASSIGN
         PacketParser := Protocol.CreatePacketParser( yyv[yysp-3], StrToInt( yyv[yysp-1]));
         
       end;
   4 : begin
         yyval := yyv[yysp-6];
       end;
   5 : begin
         yyval := yyv[yysp-1];
       end;
   6 : begin
       end;
   7 : begin
         yyval := yyv[yysp-1];
       end;
   8 : begin
         
         // matcheroption : CONDITIONAL IDENTIFIER COMPARE NUMBER CHOOSE
         Condition  := TCondition.Create( yyv[yysp-3], StrToInt( yyv[yysp-1]), False);
         IsOptional := False;
         
       end;
   9 : begin
         
         // matcheroption : CONDITIONAL IDENTIFIER NOT_COMPARE NUMBER CHOOSE
         Condition  := TCondition.Create( yyv[yysp-3], StrToInt( yyv[yysp-1]), True);
         IsOptional := False;
         
       end;
  10 : begin
         
         // matcheroption : OPTIONAL
         Condition  := Nil;
         IsOptional := True;
         
       end;
  11 : begin
         
         // matcheroption : <empty>
         Condition  := Nil;
         IsOptional := False;
         
       end;
  12 : begin
         
         // matcherType : IDENTIFIER BIND IDENTIFIER
         PacketParser.
         AddPacketMatcher(
         '',
         yyv[yysp-2],
         yyv[yysp-0],
         Condition,
         IsOptional
         );
         Condition := Nil;
         
       end;
  13 : begin
         
         // matcherType : IDENTIFIER TIMES IDENTIFIER BIND IDENTIFIER
         PacketParser.
         AddPacketMatcher(
         yyv[yysp-4],
         yyv[yysp-2],
         yyv[yysp-0],
         Condition,
         IsOptional
         );
         Condition := Nil;
         
       end;
  14 : begin
         
         // matcherType : IDENTIFIER SIZE NUMBER
         PacketParser.
         AddVariableMatcher(
         1,
         yyv[yysp-2],
         StrToInt( yyv[yysp-0]),
         0,
         Condition,
         IsOptional
         );
         Condition := Nil;
         
       end;
  15 : begin
         
         // matcherType : NUMBER TIMES IDENTIFIER SIZE NUMBER STOP NUMBER
         PacketParser.
         AddVariableMatcher(
         StrToInt( yyv[yysp-6]),
         yyv[yysp-4],
         StrToInt( yyv[yysp-2]),
         StrToInt( yyv[yysp-0]),
         Condition,
         IsOptional
         );
         Condition := Nil;
         
       end;
  16 : begin
         
         // matcherType : NUMBER SIZE NUMBER
         PacketParser.
         AddConstantMatcher(
         StrToInt( yyv[yysp-2]),
         StrtoInt( yyv[yysp-0]),
         Condition,
         IsOptional
         );
         Condition := Nil;
         
       end;

    End;
  End; // yyAction

  // ///////////////////////////////////////////////////////////////////////////

// parse table:


type YYARec = record
                sym, act : Integer;
              end;
     YYRRec = record
                len, sym : Integer;
              end;

const

yynacts   = 44;
yyngotos  = 12;
yynstates = 42;
yynrules  = 16;

yya : array [1..yynacts] of YYARec = (
{ 0: }
  ( sym: 258; act: 3 ),
  ( sym: 0; act: -2 ),
{ 1: }
  ( sym: 258; act: 3 ),
  ( sym: 0; act: -2 ),
{ 2: }
  ( sym: 0; act: 0 ),
{ 3: }
  ( sym: 259; act: 5 ),
{ 4: }
{ 5: }
  ( sym: 257; act: 6 ),
{ 6: }
  ( sym: 260; act: 7 ),
{ 7: }
{ 8: }
  ( sym: 264; act: 12 ),
  ( sym: 270; act: 13 ),
  ( sym: 269; act: -6 ),
  ( sym: 257; act: -11 ),
  ( sym: 258; act: -11 ),
{ 9: }
  ( sym: 257; act: 15 ),
  ( sym: 258; act: 16 ),
{ 10: }
  ( sym: 264; act: 12 ),
  ( sym: 270; act: 13 ),
  ( sym: 269; act: -6 ),
  ( sym: 257; act: -11 ),
  ( sym: 258; act: -11 ),
{ 11: }
  ( sym: 269; act: 18 ),
{ 12: }
{ 13: }
  ( sym: 258; act: 19 ),
{ 14: }
{ 15: }
  ( sym: 262; act: 20 ),
  ( sym: 268; act: 21 ),
{ 16: }
  ( sym: 262; act: 22 ),
  ( sym: 267; act: 23 ),
  ( sym: 268; act: 24 ),
{ 17: }
{ 18: }
{ 19: }
  ( sym: 265; act: 25 ),
  ( sym: 266; act: 26 ),
{ 20: }
  ( sym: 258; act: 27 ),
{ 21: }
  ( sym: 257; act: 28 ),
{ 22: }
  ( sym: 258; act: 29 ),
{ 23: }
  ( sym: 258; act: 30 ),
{ 24: }
  ( sym: 257; act: 31 ),
{ 25: }
  ( sym: 257; act: 32 ),
{ 26: }
  ( sym: 257; act: 33 ),
{ 27: }
  ( sym: 268; act: 34 ),
{ 28: }
{ 29: }
  ( sym: 267; act: 35 ),
{ 30: }
{ 31: }
{ 32: }
  ( sym: 261; act: 36 ),
{ 33: }
  ( sym: 261; act: 37 ),
{ 34: }
  ( sym: 257; act: 38 ),
{ 35: }
  ( sym: 258; act: 39 ),
{ 36: }
{ 37: }
{ 38: }
  ( sym: 263; act: 40 ),
{ 39: }
{ 40: }
  ( sym: 257; act: 41 )
{ 41: }
);

yyg : array [1..yyngotos] of YYARec = (
{ 0: }
  ( sym: -3; act: 1 ),
  ( sym: -2; act: 2 ),
{ 1: }
  ( sym: -3; act: 1 ),
  ( sym: -2; act: 4 ),
{ 2: }
{ 3: }
{ 4: }
{ 5: }
{ 6: }
{ 7: }
  ( sym: -5; act: 8 ),
{ 8: }
  ( sym: -7; act: 9 ),
  ( sym: -6; act: 10 ),
  ( sym: -4; act: 11 ),
{ 9: }
  ( sym: -8; act: 14 ),
{ 10: }
  ( sym: -7; act: 9 ),
  ( sym: -6; act: 10 ),
  ( sym: -4; act: 17 )
{ 11: }
{ 12: }
{ 13: }
{ 14: }
{ 15: }
{ 16: }
{ 17: }
{ 18: }
{ 19: }
{ 20: }
{ 21: }
{ 22: }
{ 23: }
{ 24: }
{ 25: }
{ 26: }
{ 27: }
{ 28: }
{ 29: }
{ 30: }
{ 31: }
{ 32: }
{ 33: }
{ 34: }
{ 35: }
{ 36: }
{ 37: }
{ 38: }
{ 39: }
{ 40: }
{ 41: }
);

yyd : array [0..yynstates-1] of Integer = (
{ 0: } 0,
{ 1: } 0,
{ 2: } 0,
{ 3: } 0,
{ 4: } -1,
{ 5: } 0,
{ 6: } 0,
{ 7: } -3,
{ 8: } 0,
{ 9: } 0,
{ 10: } 0,
{ 11: } 0,
{ 12: } -10,
{ 13: } 0,
{ 14: } -7,
{ 15: } 0,
{ 16: } 0,
{ 17: } -5,
{ 18: } -4,
{ 19: } 0,
{ 20: } 0,
{ 21: } 0,
{ 22: } 0,
{ 23: } 0,
{ 24: } 0,
{ 25: } 0,
{ 26: } 0,
{ 27: } 0,
{ 28: } -16,
{ 29: } 0,
{ 30: } -12,
{ 31: } -14,
{ 32: } 0,
{ 33: } 0,
{ 34: } 0,
{ 35: } 0,
{ 36: } -8,
{ 37: } -9,
{ 38: } 0,
{ 39: } -13,
{ 40: } 0,
{ 41: } -15
);

yyal : array [0..yynstates-1] of Integer = (
{ 0: } 1,
{ 1: } 3,
{ 2: } 5,
{ 3: } 6,
{ 4: } 7,
{ 5: } 7,
{ 6: } 8,
{ 7: } 9,
{ 8: } 9,
{ 9: } 14,
{ 10: } 16,
{ 11: } 21,
{ 12: } 22,
{ 13: } 22,
{ 14: } 23,
{ 15: } 23,
{ 16: } 25,
{ 17: } 28,
{ 18: } 28,
{ 19: } 28,
{ 20: } 30,
{ 21: } 31,
{ 22: } 32,
{ 23: } 33,
{ 24: } 34,
{ 25: } 35,
{ 26: } 36,
{ 27: } 37,
{ 28: } 38,
{ 29: } 38,
{ 30: } 39,
{ 31: } 39,
{ 32: } 39,
{ 33: } 40,
{ 34: } 41,
{ 35: } 42,
{ 36: } 43,
{ 37: } 43,
{ 38: } 43,
{ 39: } 44,
{ 40: } 44,
{ 41: } 45
);

yyah : array [0..yynstates-1] of Integer = (
{ 0: } 2,
{ 1: } 4,
{ 2: } 5,
{ 3: } 6,
{ 4: } 6,
{ 5: } 7,
{ 6: } 8,
{ 7: } 8,
{ 8: } 13,
{ 9: } 15,
{ 10: } 20,
{ 11: } 21,
{ 12: } 21,
{ 13: } 22,
{ 14: } 22,
{ 15: } 24,
{ 16: } 27,
{ 17: } 27,
{ 18: } 27,
{ 19: } 29,
{ 20: } 30,
{ 21: } 31,
{ 22: } 32,
{ 23: } 33,
{ 24: } 34,
{ 25: } 35,
{ 26: } 36,
{ 27: } 37,
{ 28: } 37,
{ 29: } 38,
{ 30: } 38,
{ 31: } 38,
{ 32: } 39,
{ 33: } 40,
{ 34: } 41,
{ 35: } 42,
{ 36: } 42,
{ 37: } 42,
{ 38: } 43,
{ 39: } 43,
{ 40: } 44,
{ 41: } 44
);

yygl : array [0..yynstates-1] of Integer = (
{ 0: } 1,
{ 1: } 3,
{ 2: } 5,
{ 3: } 5,
{ 4: } 5,
{ 5: } 5,
{ 6: } 5,
{ 7: } 5,
{ 8: } 6,
{ 9: } 9,
{ 10: } 10,
{ 11: } 13,
{ 12: } 13,
{ 13: } 13,
{ 14: } 13,
{ 15: } 13,
{ 16: } 13,
{ 17: } 13,
{ 18: } 13,
{ 19: } 13,
{ 20: } 13,
{ 21: } 13,
{ 22: } 13,
{ 23: } 13,
{ 24: } 13,
{ 25: } 13,
{ 26: } 13,
{ 27: } 13,
{ 28: } 13,
{ 29: } 13,
{ 30: } 13,
{ 31: } 13,
{ 32: } 13,
{ 33: } 13,
{ 34: } 13,
{ 35: } 13,
{ 36: } 13,
{ 37: } 13,
{ 38: } 13,
{ 39: } 13,
{ 40: } 13,
{ 41: } 13
);

yygh : array [0..yynstates-1] of Integer = (
{ 0: } 2,
{ 1: } 4,
{ 2: } 4,
{ 3: } 4,
{ 4: } 4,
{ 5: } 4,
{ 6: } 4,
{ 7: } 5,
{ 8: } 8,
{ 9: } 9,
{ 10: } 12,
{ 11: } 12,
{ 12: } 12,
{ 13: } 12,
{ 14: } 12,
{ 15: } 12,
{ 16: } 12,
{ 17: } 12,
{ 18: } 12,
{ 19: } 12,
{ 20: } 12,
{ 21: } 12,
{ 22: } 12,
{ 23: } 12,
{ 24: } 12,
{ 25: } 12,
{ 26: } 12,
{ 27: } 12,
{ 28: } 12,
{ 29: } 12,
{ 30: } 12,
{ 31: } 12,
{ 32: } 12,
{ 33: } 12,
{ 34: } 12,
{ 35: } 12,
{ 36: } 12,
{ 37: } 12,
{ 38: } 12,
{ 39: } 12,
{ 40: } 12,
{ 41: } 12
);

yyr : array [1..yynrules] of YYRRec = (
{ 1: } ( len: 2; sym: -2 ),
{ 2: } ( len: 0; sym: -2 ),
{ 3: } ( len: 0; sym: -5 ),
{ 4: } ( len: 7; sym: -3 ),
{ 5: } ( len: 2; sym: -4 ),
{ 6: } ( len: 0; sym: -4 ),
{ 7: } ( len: 2; sym: -6 ),
{ 8: } ( len: 5; sym: -7 ),
{ 9: } ( len: 5; sym: -7 ),
{ 10: } ( len: 1; sym: -7 ),
{ 11: } ( len: 0; sym: -7 ),
{ 12: } ( len: 3; sym: -8 ),
{ 13: } ( len: 5; sym: -8 ),
{ 14: } ( len: 3; sym: -8 ),
{ 15: } ( len: 7; sym: -8 ),
{ 16: } ( len: 3; sym: -8 )
);


Const

  _error = 256; // error token

  Function yyact( state, sym: Integer; Var act: Integer): Boolean;
  // search action table
  Var
    k : Integer;
  Begin
    k := yyal[ state];
    while ( k <= yyah[ state]) And ( yya[ k].sym <> sym) Do
      Inc( k);
    If k > yyah[ state]
    Then Result := false
    Else Begin
      act    := yya[ k].act;
      Result := true;
    End;
  End;


  Function yygoto( state, sym: Integer; Var nstate: Integer): Boolean;
  // search goto table
  Var
    k : Integer;
  Begin
    k := yygl[ state];
    While ( k <= yygh[ state]) And ( yyg[ k].sym <> sym) Do
      Inc( k);
    If k > yygh[ state]
    Then Result := false
    Else Begin
      nstate := yyg[k].act;
      Result := true;
    End;
  End;


  Label

    parse,
    next,
    error,
    errlab,
    shift,
    reduce,
    accept,
    abort;

  Begin // yyparse

    // initialize:

    yystate := 0; yychar := -1; yynerrs := 0; yyerrflag := 0; yysp := 0;

  parse:

    // push state and value:

    inc( yysp);
    If yysp > yymaxdepth
    Then
    Begin
      ParseError( 'yyparse stack overflow');
      Goto abort;
    end;
    yys[ yysp] := yystate;
    yyv[ yysp] := yyval;

  next:

    If ( yyd[ yystate] = 0) And ( yychar = -1)
    Then // get next symbol
    Begin
      yychar := GetToken;
      If yychar < 0
      Then yychar := 0;
    End;

    yyDebugOutFmt(
      'state %d, char %d, ''%s''',
      [
        yystate,
        yychar,
        Printable( yychar)
      ]
    );

    // determine parse action:

    yyn := yyd[ yystate];
    If yyn <> 0
    Then Goto reduce; // simple state

    // no default action; search parse table

    If Not yyact( yystate, yychar, yyn)
    Then Goto error
    Else If yyn > 0
    Then Goto shift
    Else If yyn < 0
    Then Goto reduce
    Else Goto accept;

  error:

  // error; start error recovery:

    If yyerrflag = 0
    Then ParseError( 'syntax error');

  errlab:

    If yyerrflag = 0
    Then inc(yynerrs);     // new error

    If yyerrflag<=2
    Then Begin             // incomplete recovery; try again
      yyerrflag := 3;

      // uncover a state with shift action on error token
      While
        ( yysp > 0) And Not ( yyact( yys[ yysp], _error, yyn) And ( yyn > 0))
      Do Begin
        If yydebug
        Then
          If yysp > 1
          Then
            yyDebugOutFmt(
              'error recovery pops state %d, uncovers %d',
              [
                yys[ yysp],
                yys[ yysp - 1]
              ]
            )
          Else
            yyDebugOut( 'error ecovery fails ... abort');
        Dec( yysp);
      End;
      If yysp = 0
      Then Goto abort;   // parser has fallen from stack; abort
      yystate := yyn;    // simulate shift on error
      Goto parse;
    End
    Else Begin // no shift yet; discard symbol
      yyDebugOutFmt(
        'error recovery discards char %d (%s)',
        [
          yychar,
          Printable( yychar)
        ]
      );
      If yychar = 0
      Then Goto abort;           // end of input; abort
      yychar := -1;
      Goto next;                 // clear lookahead char and try again
    End;

  shift:

    // go to new state, clear lookahead character:

    yystate := yyn;
    yychar  := -1;
    yyval   := yylval;
    If yyerrflag > 0
    Then Dec( yyerrflag);

    Goto parse;

  reduce:

    // execute action, pop rule from stack, and go to next state:

    yyDebugOutFmt( 'reduce %d', [ -yyn]);

    yyflag := yyfnone;
    yyaction( -yyn);
    dec( yysp, yyr[ -yyn].len);
    If yygoto( yys[ yysp], yyr[ -yyn].sym, yyn)
    Then yystate := yyn;

    // handle action calls to yyaccept, yyabort and yyerror:

    Case yyflag Of
      yyfaccept : goto accept;
      yyfabort  : goto abort;
      yyferror  : goto errlab;
    End;

    Goto parse;

  accept:

    Result := 0;
    exit;

  abort:

    Result := 1;
    exit;

End; // yyparse



// /////////////////////////////////////////////////////////////////////////////
// Include the lexer

{$INCLUDE pdllex.pas}

// End  of lexer.
// /////////////////////////////////////////////////////////////////////////////


End.
