%{

// /////////////////////////////////////////////////////////////////////////////
//
//  Protocol Definition Language
//  Copyright (C) 2003 Marcus Andersson
//
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
//
//  ////////////////////////////////////////////////////////////////////////////
//
//  Modified to be used in Delphi 7 by Jan Punter.
//
//  Modifications CopyRight (c) 2003 Jan Punter
//
//  Distributed under the same conditions as the original code by Narcus
//  Andersson.
//
//  This should be compileable for the tp yacc vession 3.0 as taken from
//  the free pascal distribution. Modified versions of lexlib.pas, yacclib.pas,
//  yyylex.cod and yyparse.cod were used. These should be present in this
//  distribution.
//
//  Notes :
//
//    Take care with quoting in the comments above (and below), an uneven number
//    of quotes in this section will give compile errors from pyacc.
//
//    The lexical analyzer created from pdllex.l is included in the code
//    generated from this file, and it _must_ be included.
//
//    The word END was used as a terminal token, this had to be changed as
//    END is a reserved word in Pascal. I changed it into _END.
//
//    lupd : 2003-10-13
//
//  ////////////////////////////////////////////////////////////////////////////



Unit pdlparser;

Interface

Uses

  SysUtils, Classes, pdlstructs;


Type

  YYSType = String;

Var

  Protocol     : TProtocol;
  PacketParser : TPacketParser;
  Condition    : TCondition;
  IsOptional   : Boolean;      // Optinal clashed with 'OPTIONAL', Pascal is not
                               // case sensitive. Renamed to IsOptional.

%}

%token NUMBER
%token IDENTIFIER
%token PAD
%token ASSIGN
%token CHOOSE
%token TIMES
%token STOP
%token OPTIONAL      /* Also see IsOptional, above       */
%token COMPARE
%token NOT_COMPARE
%token BIND
%token SIZE
%token _END          /* END is a reserved word in Pascal */
%token CONDITIONAL


%%

protocol
  : packet protocol
  |
	;

packet
  : IDENTIFIER PAD NUMBER ASSIGN
    {
      // packet : IDENTIFIER PAD NUMBER ASSIGN
      PacketParser := Protocol.CreatePacketParser( $1, StrToInt( $3));
    }
    matchers _END
	;

matchers
  : matcher matchers
  |
	;

matcher
  : matcherOption matcherType
	;

matcherOption
	: CONDITIONAL IDENTIFIER COMPARE NUMBER CHOOSE
    {
      // matcheroption : CONDITIONAL IDENTIFIER COMPARE NUMBER CHOOSE
      Condition  := TCondition.Create( $2, StrToInt( $4), False);
      IsOptional := False;
    }
  | CONDITIONAL IDENTIFIER NOT_COMPARE NUMBER CHOOSE
    {
      // matcheroption : CONDITIONAL IDENTIFIER NOT_COMPARE NUMBER CHOOSE
      Condition  := TCondition.Create( $2, StrToInt( $4), True);
      IsOptional := False;
    }
  | OPTIONAL
    {
      // matcheroption : OPTIONAL
      Condition  := Nil;
      IsOptional := True;
    }
  |
    {
      // matcheroption : <empty>
      Condition  := Nil;
      IsOptional := False;
    }
  ;

matcherType
  : IDENTIFIER BIND IDENTIFIER
    {
      // matcherType : IDENTIFIER BIND IDENTIFIER
      PacketParser.
        AddPacketMatcher(
          '',
          $1,
          $3,
          Condition,
          IsOptional
        );
        Condition := Nil;
    }
  | IDENTIFIER TIMES IDENTIFIER BIND IDENTIFIER
    {
      // matcherType : IDENTIFIER TIMES IDENTIFIER BIND IDENTIFIER
      PacketParser.
        AddPacketMatcher(
          $1,
          $3,
          $5,
          Condition,
          IsOptional
        );
        Condition := Nil;
    }
  | IDENTIFIER SIZE NUMBER
    {
      // matcherType : IDENTIFIER SIZE NUMBER
      PacketParser.
        AddVariableMatcher(
          1,
          $1,
          StrToInt( $3),
          0,
          Condition,
          IsOptional
        );
        Condition := Nil;
    }
  | NUMBER TIMES IDENTIFIER SIZE NUMBER STOP NUMBER
    {
      // matcherType : NUMBER TIMES IDENTIFIER SIZE NUMBER STOP NUMBER
      PacketParser.
        AddVariableMatcher(
          StrToInt( $1),
          $3,
          StrToInt( $5),
          StrToInt( $7),
          Condition,
          IsOptional
        );
        Condition := Nil;
    }
  | NUMBER SIZE NUMBER
    {
      // matcherType : NUMBER SIZE NUMBER
      PacketParser.
        AddConstantMatcher(
          StrToInt( $1),
          StrtoInt( $3),
          Condition,
          IsOptional
        );
        Condition := Nil;
    }
	;

%%

// /////////////////////////////////////////////////////////////////////////////
// Include the lexer

{$INCLUDE pdllex.pas}

// End  of lexer.
// /////////////////////////////////////////////////////////////////////////////


End.

