Unit pdlstructs;

//  ////////////////////////////////////////////////////////////////////////////
//
//  Protocol Definition Language
//  Copyright (C) 2003 Marcus Andersson
//
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
//
//  ////////////////////////////////////////////////////////////////////////////
//
//  Translated into Pascal by Jan Punter, meant to be compiled under Delphi 7.
//
//  Modifications CopyRight (c) 2003 Jan Punter
//
//  Distributed under the same conditions as the original code by Narcus
//  Andersson.
//
//    lupd : 2003-10-13
//
//  ////////////////////////////////////////////////////////////////////////////


Interface

Uses

  SysUtils, Classes, Globals;

Type

  TPacket       = Class;
  TPacketList   = Class;
  TVariableList = Class;
  TMatcher      = Class;
  TProtocol     = Class;
  TPacketParser = Class;

  PdlException = Class( Exception)
  Private
    FError : Integer;
  Public
    Constructor Create( Const aMsg: String; anError: Integer);
  Public
    Property    Error : Integer Read FError;
  End;

  TTracer = Class
  Public
    Procedure   Trace( Const aMsg: String);                             Virtual;
    Procedure   TraceFmt( Const aFormat: String; Const anArgs: Array Of Const);
  End;

  TBitStream = Class
  Private
    FBits     : Array Of Byte;
    FPosition : Integer;
  Private
    Function    GetSize: Integer;
    Procedure   SetSize( aValue: Integer);
  Public
    Constructor Create;
    Destructor  Destroy;                                               Override;
    Procedure   Reset;
    Procedure   Append( aData, aSize: Integer);
    Function    HasAvailable( aSize: Integer): Boolean;
    Function    GetInt( aSize: Integer): Integer;
  Public
    Property    Size    : Integer Read GetSize     Write SetSize;
    Property    Position: Integer Read FPosition Write FPosition;
  End;

  TIntStream = Class
  Private
    FInts     : Array Of Integer;
    FPosition : Integer;
  Private
    Function    GetSize: Integer;
    Procedure   SetSize( aValue: Integer);
  Public
    Constructor Create;
    Destructor  Destroy;                                               Override;
    Procedure   Reset;
    Procedure   Append( aData: Integer);
    Function    HasAvailable( aSize: Integer): Boolean;
    Function    GetInt: Integer;
  Public
    Property    Size    : Integer Read GetSize     Write SetSize;
    Property    Position: Integer Read FPosition Write FPosition;
  End;

  TVariableList = Class
  Private
    FData : Array Of Integer;
  Private
    Function    GetSize: Integer;
    Procedure   SetSize( aValue: Integer);
    Function    GetItem( anIndex: Integer): Integer;
  Public
    Constructor Create;
    Destructor  Destroy;                                               Override;
    Procedure   Clear;
    Procedure   Push( aValue: Integer);
    Function    Head: Integer;
  Public
    Property    Size: Integer Read GetSize Write SetSize;
    Property    Items[ anIndex: Integer]: Integer Read GetItem;
  End;

  TPdlObjectList = Class
  Private
    FData : Array Of TObject;
  Private
    Function    GetSize: Integer;
    Procedure   SetSize( aValue: Integer);
    Function    GetItem( anIndex: Integer): TObject;
  Public
    Constructor Create;
    Destructor  Destroy;                                               Override;
    Procedure   Clear;
    Procedure   Push( aValue: TObject);
    Function    Head: TObject;
    Function    Remove( anObject: TObject): TObject;
  Public
    Property    Size: Integer Read GetSize Write SetSize;
    Property    Items[ anIndex: Integer]: TObject Read GetItem;
  End;

  TPacketList = Class( TPdlObjectList)
  Private
    Function    GetItem( anIndex: Integer): TPacket;
  Public
    Procedure   Push( aValue: TPacket);
    Function    Head: TPacket;
    Function    Contains( Const aPacketName: String): Boolean;
  Public
    Property    Items[ anIndex: Integer]: TPacket Read GetItem;
  End;

  TMatcherList = Class( TPdlObjectList)
  Private
    Function    GetItem( anIndex: Integer): TMatcher;
  Public
    Procedure   Push( aValue: TMatcher);
    Function    Head: TMatcher;
  Public
    Property    Items[ anIndex: Integer]: TMatcher Read GetItem;
  End;

  TVariableMap = Class( TStringList)
  Private
    Function    GetData( Const anIndex: String): Integer;
    Procedure   SetData( Const anIndex: String; aValue: Integer);
  Public
    Constructor Create;
  Public
    Property    Data[ Const anIndex: String]: Integer Read GetData Write SetData; Default;
  End;

  TObjectMap = Class( TStringList)
  Private
    Function    GetData( Const anIndex: String): TObject;
    Procedure   SetData( Const anIndex: String; aValue: TObject);
  Public
    Constructor Create;
    Destructor  Destroy;                                               Override;
  Public
    Property    Data[ Const anIndex: String]: TObject Read GetData Write SetData; Default;
  End;

  TPacketMap = Class( TObjectMap)
  Private
    Function    GetData( Const anIndex: String): TPacket;
    Procedure   SetData( Const anIndex: String; aValue: TPacket);
  Public
    Function    Contains( Const aPacketName: String): Boolean;
  Public
    Property    Data[ Const anIndex: String]: TPacket Read GetData Write SetData; Default;
  End;

  TVariableListMap = Class( TObjectMap)
  Private
    Function    GetData( Const anIndex: String): TVariableList;
    Procedure   SetData( Const anIndex: String; aValue: TVariableList);
  Public
    Property    Data[ Const anIndex: String]: TVariableList Read GetData Write SetData; Default;
  End;

  TPacketListMap = Class( TObjectMap)
  Private
    Function    GetData( Const anIndex: String): TPacketList;
    Procedure   SetData( Const anIndex: String; aValue: TPacketList);
  Public
    Function    Contains( Const aPacketName: String): Boolean;
  Public
    Property    Data[ Const anIndex: String]: TPacketList Read GetData Write SetData; Default;
  End;

  TPacketParserMap = Class( TObjectMap)
  Private
    Function    GetData( Const anIndex: String): TPacketParser;
    Procedure   SetData( Const anIndex: String; aValue: TPacketparser);
  Public
    Property    Data[ Const anIndex: String]: TPacketParser Read GetData Write SetData; Default;
  End;

  TPacket = Class
  Private
    FName         : String;
    FVariables    : TVariableMap;
    FAllVariables : TVariableList;
    FPackets      : TPacketMap;
    FPacketLists  : TPacketListMap;
    FIntLists     : TVariableListMap;
  Private
    Function    GetPacket      ( Const anIndex: String): TPacket;
    Function    GetVariable    ( Const anIndex: String): Integer;
    Function    GetPacketList  ( Const anIndex: String): TPacketList;
    Function    GetVariableList( Const anIndex: String): TVariableList;
  Public
    Constructor Create;
    Destructor  Destroy;                                               Override;
    Procedure   Clear;
    Function    Contains( Const aPacketName: String): Boolean;
  Public
    Procedure   Bind( aValue: TPacket      ; Const aName: String);     Overload;
    Procedure   Bind( aValue: Integer      ; Const aName: String);     Overload;
    Procedure   Bind( aValue: TPacketList  ; Const aName: String);     Overload;
    Procedure   Bind( aValue: TVariableList; Const aName: String);     Overload;
  Public
    Property    Name        : String        Read FName Write FName;
    Property    AllVariables: TVariableList Read FAllVariables;
  Public
    Property    Packet      [ Const anIndex: String]: TPacket       Read Getpacket;
    Property    Variable    [ Const anIndex: String]: Integer       Read getVariable;
    Property    PacketList  [ Const anIndex: String]: TPacketList   Read GetPacketList;
    Property    VariableList[ Const anIndex: String]: TVariableList Read GetVariableList;
  End;

  TCondition = Class
  Private
    FVariable : String;
    FValue    : Integer;
    FNegate   : Boolean;
  Public
    Constructor Create( Const aVariable: String; aValue: Integer; aNegate: Boolean);
    Function    Eval( aPacket: TPacket): Boolean;
  End;

  TMatcher = Class
  Private
    FOptional  : Boolean;
    FCondition : TCondition;
  Public
    Constructor Create( aCondition: TCondition; anOptional: Boolean);
    Destructor  Destroy;                                               Override;
    Function    IsConditional: Boolean;
    Function    IsOptional   : Boolean;
    Function    TrueCondition( aPacket: TPacket): Boolean;
    Function    Match(
      aProtocol : TProtocol;
      aData     : TBitStream;
      aPacket   : TPacket
    ): Boolean;                                               Virtual; Abstract;
    Function    Apply(
      aProtocol  : TProtocol;
      aPacket    : TPacket;
      aData      : TIntStream;
      aBitStream : TBitStream
    ): Boolean;                                               Virtual; Abstract;
  End;

  TConstantMatcher = Class( TMatcher)
  Private
    FValue : Integer;
    FSize  : Integer;
  Private
    Procedure   Trace( aProtocol: TProtocol);
  Public
    Constructor Create( aValue, aSize: Integer; aCondition: TCondition; anOptional: Boolean);
    Function    Match(
      aProtocol : TProtocol;
      aData     : TBitStream;
      aPacket   : TPacket
    ): Boolean;                                                        Override;
    Function    Apply(
      aProtocol  : TProtocol;
      aPacket    : TPacket;
      aData      : TIntStream;
      aBitStream : TBitStream
    ): Boolean;                                                        Override;
  End;

  TVariableMatcher = Class( TMatcher)
  Private
    FVariable : String;
    FCount    : Integer;
    FSize     : Integer;
    FTerminal : Integer;
  Private
    Procedure   Trace( aProtocol: TProtocol);
  Public
    Constructor Create(
      aCount          : Integer;
      Const aVariable : String;
      aSize           : Integer;
      aTerminal       : Integer;
      aCondition      : TCondition;
      anOptional      : Boolean
    );
    Function    Match(
      aProtocol : TProtocol;
      aData     : TBitStream;
      aResult   : TPacket
    ): Boolean;                                                        Override;
    Function    Apply(
      aProtocol : TProtocol;
      aPacket   : TPacket;
      aData     : TIntStream;
      aResult   : TBitStream
    ): Boolean;                                                        Override;
  End;

  TPacketMatcher = Class( TMatcher)
  Private
    FCount      : String;
    FParserName : String;
    FBinding    : String;
  Private
    Procedure   Trace( aProtocol: TProtocol);
  Public
    Constructor Create(
      Const aCount      : String;
      Const aParserName : String;
      Const aBinding    : String;
      aCondition        : TCondition;
      anOptional        : Boolean
    );
    Function    Match(
      aProtocol : TProtocol;
      aData     : TBitStream;
      aResult   : TPacket
    ): Boolean;                                                        Override;
    Function    Apply(
      aProtocol : TProtocol;
      aPacket   : TPacket;
      aData     : TIntStream;
      aResult   : TBitStream
    ): Boolean;                                                        Override;
  End;

  TPacketParser = Class
  Private
    FName     : String;
    FPadding  : Integer;
    FProtocol : TProtocol;
    FMatchers : TMatcherList;
  Public
    Constructor Create( Const aName: String; aPadding: Integer; aProtocol: TProtocol);
    Destructor  Destroy;                                               Override;
    Procedure   AddPacketMatcher(
      Const aCount      : String;
      Const aPacketName : String;
      Const aBinding    : String;
      aCondition        : TCondition;
      anOptional        : Boolean
    );
    Procedure   AddVariableMatcher(
      aCount     : Integer;
      aVariable  : String;
      aSize      : Integer;
      aTerminal  : Integer;
      aCondition : TCondition;
      anOptional : Boolean
    );
    Procedure   AddConstantMatcher(
      aConstant  : Integer;
      aSize      : Integer;
      aCondition : TCondition;
      anOptional : Boolean
    );
    Function    Parse( aData: TBitStream; aResult: TPacket): Boolean;
    Function    Generate( aData: TIntStream; aResult: TBitStream): Boolean;
  End;

  TProtocol = Class
  Private
    FTracer        : TTracer;
    FPacketParsers : TPacketParserMap;
  Private
    Function    GetPacketParser( Const anIndex: String): TPacketParser;
    Procedure   DoParseError( aSender: TObject; Const aMsg: String; Var AbortParsing: Boolean);
  Public
    Constructor Create;
    Destructor  Destroy;                                               Override;
    Procedure   LoadPdlFile( Const aFileName: String; DoYaccDebug: Boolean = False);
    Procedure   UseTracer( aTracer: TTracer);
    Procedure   Trace( Const aMsg: String);
    Procedure   TraceFmt( Const aFormat: String; Const anArgs: Array Of Const);
    Function    CreatePacketParser( Const aName: String; aPadding: Integer): TPacketParser;
  Public
    Property    PacketParser[ Const anIndex: String]: TPacketparser Read GetPacketparser;
  End;



Implementation

Uses

  LexLib, pdlParser;


{ ========
  PdlException = Class( Exception)
  Private
    FError : Integer;
  Public
    Property    Error : Integer Read FError;
  Public
}

    Constructor PdlException.Create( Const aMsg: String; anError: Integer);
    Begin
      Inherited Create( aMsg);
      FError := anError;
    End;

{ ========
  TTracer = Class
  Public
}

    Procedure   TTracer.Trace( Const aMsg: String); // Virtual;
    Begin
    End;

    Procedure   TTracer.TraceFmt( Const aFormat: String; Const anArgs: Array Of Const);
    Begin
      Trace( Format( aFormat, anArgs));
    End;


{ ========
  TBitStream = Class
  Private
    FBits     : Array Of Byte;
    FPosition : Integer;
  Public
    Property    Size    : Integer Read GetSize     Write SetSize;
    Property    Position: Integer Read GetPosition Write SetPosition;
  Private
}

    Function    TBitStream.GetSize: Integer;
    Begin
      Result := Length( FBits);
    End;

    Procedure   TBitStream.SetSize( aValue: Integer);
    Begin
      SetLength( FBits, aValue);
      If Position > Size
      Then Position := Size;
    End;

//  Public

    Constructor TBitStream.Create;
    Begin
      Inherited;
      Size := 0;
      Reset;
    End;

    Destructor  TBitStream.Destroy; // Override;
    Begin
      Size := 0;
      Inherited;
    End;

    Procedure   TBitStream.Reset;
    Begin
      Position := 0;
    End;

    Procedure   TBitStream.Append( aData, aSize: Integer);
    Var
      i : Integer;
    Begin
      Size := Size + aSize;
      For i := aSize - 1 DownTo 0 Do
        FBits[ Size - i - 1] := ( aData Shr i) And 1;
    End;

    Function    TBitStream.HasAvailable( aSize: Integer): Boolean;
    Begin
      Result := Position + aSize <= Size;
    End;

    Function    TBitStream.GetInt( aSize: Integer): Integer;
    Var
      i : Integer;
    Begin
      Result := 0;
      For i := 0 To aSize - 1 Do
      Begin
        Result   := ( Result Shl 1) + FBits[ Position];
        Position := Position + 1;
      End;
    End;


{ ========
  TIntStream = Class
  Private
    FInts     : Array Of Integer;
    FPosition : Integer;
  Public
    Property    Size    : Integer Read GetSize     Write SetSize;
    Property    Position: Integer Read FPosition Write FPosition;
  Private
}

    Function    TIntStream.GetSize: Integer;
    Begin
      Result := Length( FInts);
    End;

    Procedure   TIntStream.SetSize( aValue: Integer);
    Begin
      SetLength( FInts, aValue);
      If Position > Size
      Then Position := Size;
    End;

//  Public

    Constructor TIntStream.Create;
    Begin
      Inherited;
      Size := 0;
      Reset;
    End;

    Destructor  TIntStream.Destroy; // Override;
    Begin
      Size := 0;
      Inherited;
    End;

    Procedure   TIntStream.Reset;
    Begin
      Position := 0;
    End;

    Procedure   TIntStream.Append( aData: Integer);
    Begin
      Size := Size + 1;
      FInts[ Size - 1] := aData;
    End;

    Function    TIntStream.HasAvailable( aSize: Integer): Boolean;
    Begin
      Result := Position + aSize <= Size;
    End;

    Function    TIntStream.GetInt: Integer;
    Begin
      Result := FInts[ Position];
      Position := Position + 1;
    End;



{ ========
  TVariableList = Class
  Private
    FData : Array Of Integer;
  Public
    Property    Size: Integer Read GetSize Write SetSize;
    Property    Items[ anIndex: Integer]: Integer Read GetItem;
  Private
}

    Function    TVariableList.GetSize: Integer;
    Begin
      Result := Length( FData);
    End;

    Procedure   TVariableList.SetSize( aValue: Integer);
    Begin
      SetLength( FData, aValue);
    End;

    Function    TVariableList.GetItem( anIndex: Integer): Integer;
    Begin
      Result := FData[ anIndex];
    End;

//  Public

    Constructor TVariableList.Create;
    Begin
      Inherited;
      Clear;
    End;

    Destructor  TVariableList.Destroy; // Override;
    Begin
      Clear;
      Inherited;
    End;

    Procedure   TVariableList.Clear;
    Begin
      Size := 0;
    End;

    Procedure   TVariableList.Push( aValue: Integer);
    Begin
      Size := Size + 1;
      FData[ Size - 1] := aValue;
    End;

    Function    TVariableList.Head: Integer;
    Begin
      Result := FData[ 0];
    End;


{ ========
  TPdlObjectList = Class
  Private
    FData : Array Of TObject;
  Public
    Property    Size: Integer Read GetSize Write SetSize;
    Property    Items[ anIndex: Integer]: TObject Read GetItem;
  Private
}

    Function    TPdlObjectList.GetSize: Integer;
    Begin
      Result := Length( FData);
    End;

    Procedure   TPdlObjectList.SetSize( aValue: Integer);
    Begin
      SetLength( FData, aValue);
    End;

    Function    TPdlObjectList.GetItem( anIndex: Integer): TObject;
    Begin
      Result := FData[ anIndex];
    End;

//  Public

    Constructor TPdlObjectList.Create;
    Begin
      Inherited;
      Clear;
    End;

    Destructor  TPdlObjectList.Destroy; // Override;
    Var
      i : Integer;
    Begin
      For i := 0 To Size - 1 Do
        FreeAndNil( FData[ i]);
      Inherited;
    End;

    Procedure   TPdlObjectList.Clear;
    Begin
      Size := 0;
    End;

    Procedure   TPdlObjectList.Push( aValue: TObject);
    Begin
      Size := Size + 1;
      FData[ Size - 1] := aValue;
    End;

    Function    TPdlObjectList.Head: TObject;
    Begin
      Result := FData[ 0];
    End;

    Function    TPdlObjectList.Remove( anObject: TObject): TObject;
    Var
      i : Integer;
      p : Integer;
    Begin
      Result := Nil;
      For i := 0 To Size - 1 Do
        If FData[ i] = anObject
        Then Begin
          If i < Size - 1
          Then Begin
            For p := i + 1 To Size - 1 Do
              FData[ i - 1] := FData[ i];
          End;
          Size   := Size - 1;
          Result := anObject;
          Break;
        End;
    End;

{ ========
  TPacketList = Class( TPdlObjectList)
  Public
    Property    Items[ anIndex: Integer]: TPacket Read GetItem;
  Private
}


    Function    TPacketList.GetItem( anIndex: Integer): TPacket;
    Begin
      Result := ( Inherited GetItem( anIndex)) As TPacket;
    End;

//  Public

    Procedure   TPacketList.Push( aValue: TPacket);
    Begin
      Inherited Push( aValue);
    End;

    Function    TPacketList.Head: TPacket;
    Begin
      Result := Inherited Head As TPacket;
    End;

    Function    TPacketList.Contains( Const aPacketName: String): Boolean;
    Var
      i : Integer;
    Begin
      Result := False;
      For i := 0 To Size - 1 Do
        If Items[ i].Contains( aPacketName)
        Then Begin
          Result := True;
          Break;
        End;
    End;

{ ========
  TMatcherList = Class( TPdlObjectList)
  Public
    Property    Items[ anIndex: Integer]: TMatcher Read GetItem;
  Private
}

    Function    TMatcherList.GetItem( anIndex: Integer): TMatcher;
    Begin
      Result := ( Inherited GetItem( anIndex)) As TMatcher;
    End;

//  Public

    Procedure   TMatcherList.Push( aValue: TMatcher);
    Begin
      Inherited Push( aValue);
    End;

    Function    TMatcherList.Head: TMatcher;
    Begin
      Result := Inherited Head As TMatcher;
    End;


{ ========
  TVariableMap = Class( TStringList)
  Public
    Property    Data[ Const anIndex: String]: Integer Read GetData Write SetData; Default;
  Private
}

    Function    TVariableMap.GetData( Const anIndex: String): Integer;
    Var
      i : Integer;
    Begin
      i := IndexOf( anIndex);
    {$WARN UNSAFE_CAST OFF}
      If i >= 0
      Then
        Result := Integer( Objects[ i])
      Else Result := -1;
    {$WARN UNSAFE_CAST ON}
    End;

    Procedure   TVariableMap.SetData( Const anIndex: String; aValue: Integer);
    Var
      i : Integer;
    Begin
      i := IndexOf( anIndex);
      If i < 0
      Then
      {$WARN UNSAFE_CAST OFF}
        AddObject( anIndex, TObject( aValue));
      {$WARN UNSAFE_CAST ON}
    End;

//  Public

    Constructor TVariableMap.Create;
    Begin
      Inherited;
      Sorted        := True;
      CaseSensitive := False;
    End;


{ ========
  TObjectMap = Class( TStringList)
  Public
    Property    Data[ Const anIndex: String]: TObject Read GetData Write SetData; Default;
  Private
}

    Function    TObjectMap.GetData( Const anIndex: String): TObject;
    Var
      i : Integer;
    Begin
      i := IndexOf( anIndex);
      If i >= 0
      Then Result := Objects[ i]
      Else Result := Nil;
    End;

    Procedure   TObjectMap.SetData( Const anIndex: String; aValue: TObject);
    Var
      i : Integer;
    Begin
      i := IndexOf( anIndex);
      If i < 0
      Then AddObject( anIndex, aValue);
    End;

//  Public

    Constructor TObjectMap.Create;
    Begin
      Inherited Create;
      Sorted        := True;
      CaseSensitive := False;
    End;

    Destructor  TObjectMap.Destroy; // Override;
    Var
      i : Integer;
    Begin
      For i := 0 To Count - 1 Do
        If Assigned( Objects[ i])
        Then Begin
          Objects[ i].Free;
          Objects[ i] := Nil;
        End;
      Inherited;
    End;


{ ========
  TPacketMap = Class( TObjectMap)
  Public
    Property    Data[ Const anIndex: String]: TPacket Read GetData Write SetData; Default;
  Private
}

    Function    TPacketMap.GetData ( Const anIndex: String): TPacket;
    Begin
      Result := Inherited GetData( anIndex) As TPacket;
    End;

    Procedure   TPacketMap.SetData( Const anIndex: String; aValue: TPacket);
    Begin
      Assert( Assigned( aValue));
      Inherited SetData( anIndex, aValue);
    End;

//  Public

    Function    TPacketMap.Contains( Const aPacketName: String): Boolean;
    Var
      i : Integer;
    Begin
      Result := False;
      For i := 0 To Count - 1 Do
      Begin
        If ( Objects[ i] As TPacket).Contains( aPacketName)
        Then Begin
          Result := True;
          Break;
        End;
      End;
    End;


{ ========
  TVariableListMap = Class( TObjectMap)
  Public
    Property    Data[ Const anIndex: String]: TVariableList Read GetData Write SetData; Default;
  Private
}

    Function    TVariableListMap.GetData( Const anIndex: String): TVariableList;
    Begin
      Result := Inherited GetData( anIndex) As TVariableList;
    End;

    Procedure   TVariableListMap.SetData( Const anIndex: String; aValue: TVariableList);
    Begin
      Inherited SetData( anIndex, aValue);
    End;


{ ========
  TPacketListMap = Class( TObjectMap)
  Public
    Property    Data[ Const anIndex: String]: TPacketList Read GetData Write SetData; Default;
  Private
}

    Function    TPacketListMap.GetData( Const anIndex: String): TPacketList;
    Begin
      Result := Inherited GetData( anIndex) As TPacketList;
    End;

    Procedure   TPacketListMap.SetData( Const anIndex: String; aValue: TPacketList);
    Begin
      Assert( Assigned( aValue));
      Inherited SetData( anIndex, aValue);
    End;

//  Public

    Function    TPacketListMap.Contains( Const aPacketName: String): Boolean;
    Var
      i : Integer;
    Begin
      Result := False;
      For i := 0 To Count - 1 Do
      Begin
        If ( Objects[ i] As TPacketList).Contains( aPacketName)
        Then Begin
          Result := True;
          Break;
        End;
      End;
    End;


{ ========
  TPacketParserMap = Class( TObjectMap)
  Public
    Property    Data[ Const anIndex: String]: TPacketParser Read GetData Write SetData; Default;
  Private
}

    Function    TPacketParserMap.GetData( Const anIndex: String): TPacketParser;
    Begin
      Result := Inherited GetData( anIndex) As TPacketparser;
    End;

    Procedure   TPacketParserMap.SetData( Const anIndex: String; aValue: TPacketparser);
    Begin
      Inherited SetData( anIndex, aValue);
    End;


{ ========
  TPacket = Class
  Private
    FName         : String;
    FVariables    : TVariableMap;
    FAllVariables : TVariableList;
    FPackets      : TPacketMap;
    FPacketLists  : TPacketListMap;
    FIntLists     : TVariableListMap;
  Public
    Property    Name        : String        Read FName Write FName;
    Property    AllVariables: TVariableList Read FAllVariables;
  Public
    Property    Packet      [ Const anIndex: String]: TPacket       Read Getpacket;
    Property    Variable    [ Const anIndex: String]: Integer       Read getVariable;
    Property    PacketList  [ Const anIndex: String]: TPacketList   Read GetPacketList;
    Property    VariableList[ Const anIndex: String]: TVariableList Read GetVariableList;
  Private
}

    Function    TPacket.GetPacket( Const anIndex: String): TPacket;
    Var
      P : Integer;
    Begin
      p := Pos( ':', anIndex);
      If p > 0
      Then Result := FPackets[ Copy( anIndex, 1, p - 1)].Packet[ Copy( anIndex, p + 1, Length( anIndex))]
      Else Result := FPackets[ anIndex];
    End;

    Function    TPacket.GetVariable( Const anIndex: String): Integer;
    Var
      P : Integer;
    Begin
      p := Pos( ':', anIndex);
      If p > 0
      Then Result := FPackets[ Copy( anIndex, 1, p - 1)].Variable[ Copy( anIndex, p + 1, Length( anIndex))]
      Else Result := FVariables[ anIndex];
    End;

    Function    TPacket.GetPacketList( Const anIndex: String): TPacketList;
    Var
      P : Integer;
    Begin
      p := Pos( ':', anIndex);
      If p > 0
      Then Result := FPackets[ Copy( anIndex, 1, p - 1)].PacketList[ Copy( anIndex, p + 1, Length( anIndex))]
      Else Result := FPacketLists[ anIndex];
    End;

    Function    TPacket.GetVariableList( Const anIndex: String): TVariableList;
    Var
      P : Integer;
    Begin
      p := Pos( ':', anIndex);
      If p > 0
      Then Result := FPackets[ Copy( anIndex, 1, p - 1)].VariableList[ Copy( anIndex, p + 1, Length( anIndex))]
      Else Result := FIntLists[ anIndex];
    End;

//  Public

    Constructor TPacket.Create;
    Begin
      Inherited Create;
      FVariables    := TVariableMap    .Create;
      FAllVariables := TVariableList   .Create;
      FPackets      := TPacketMap      .Create;
      FPacketLists  := TPacketListMap  .Create;
      FIntLists     := TVariableListMap.Create;
    End;

    Destructor  TPacket.Destroy; // Override;
    Begin
      FreeAndNil( FIntLists    );
      FreeAndNil( FPacketLists );
      FreeAndNil( FPackets     );
      FreeAndNil( FAllVariables);
      FreeAndNil( FVariables   );
      Inherited;
    End;

    Procedure   TPacket.Clear;
    Begin
      FPackets    .Clear;
      FVariables  .Clear;
      FPacketLists.Clear;
      FIntLists   .Clear;
    End;

    Function    TPacket.Contains( Const aPacketName: String): Boolean;
    Begin
      Result :=
        ( UpperCase( Name) = UpperCase( aPacketName))                      Or
        ( Assigned( FPackets    ) And FPackets    .Contains( aPacketName)) Or
        ( Assigned( FPacketLists) And FPacketLists.Contains( aPacketName));
    End;

//  Public

    Procedure   TPacket.Bind( aValue: TPacket; Const aName: String); // Overload;
    Begin
      Assert( Assigned( aValue));
      FPackets[ aName] := aValue;
    End;

    Procedure   TPacket.Bind( aValue: Integer; Const aName: String); // Overload;
    Begin
      FVariables[ aName] := aValue;
      FAllVariables.Push( aValue);
    End;

    Procedure   TPacket.Bind( aValue: TPacketList; Const aName: String); // Overload;
    Begin
      Assert( Assigned( aValue));
      FPacketLists[ aName] := aValue;
    End;

    Procedure   TPacket.Bind( aValue: TVariableList; Const aName: String); // Overload;
    Begin
      Assert( Assigned( aValue));
      FIntLists[ aName] := aValue;
    End;


{ ========
  TCondition = Class
  Private
    FVariable : String;
    FValue    : Integer;
    FNegate   : Boolean;
  Public
}

    Constructor TCondition.Create( Const aVariable: String; aValue: Integer; aNegate: Boolean);
    Begin
      Inherited Create;
      FVariable := aVariable;
      FValue    := aValue;
      FNegate   := aNegate;
    End;

    Function    TCondition.Eval( aPacket: TPacket): Boolean;
    Begin
      Result := FNegate Xor ( aPacket.Variable[ FVariable] = FValue);
    End;


{ ========
  TMatcher = Class
  Private
    FOptional  : Boolean;
    FCondition : TCondition;
  Public
    Function    Match(
      aProtocol : TProtocol;
      aData     : TBitStream;
      aPacket   : TPacket
    ): Boolean;                                               Virtual; Abstract;
    Function    Apply(
      aProtocol  : TProtocol;
      aPacket    : TPacket;
      aData      : TStream;
      aBitStream : TBitStream
    ): Boolean;                                               Virtual; Abstract;
}

    Constructor TMatcher.Create( aCondition: TCondition; anOptional: Boolean);
    Begin
      Inherited Create;
      FCondition := aCondition;
      FOptional  := anOptional;
    End;

    Destructor  TMatcher.Destroy; // Override;
    Begin
      FreeAndNil( FCondition);
      Inherited;
    End;

    Function    TMatcher.IsConditional: Boolean;
    Begin
      Result := Assigned( FCondition);
    End;

    Function    TMatcher.IsOptional: Boolean;
    Begin
      Result := FOptional;
    End;

    Function    TMatcher.TrueCondition( aPacket: TPacket): Boolean;
    Begin
      Result := IsConditional And FCondition.Eval( aPacket);
    End;


{ ========
  TConstantMatcher = Class( TMatcher)
  Private
    FValue : Integer;
    FSize  : Integer;
  Private
}

    Procedure   TConstantMatcher.Trace( aProtocol: TProtocol);
    Begin
      aProtocol.Trace( Copy( Format( '  %d:%d', [ FValue, FSize]), 1, 100));
    End;

//  Public

    Constructor TConstantMatcher.Create( aValue, aSize: Integer; aCondition: TCondition; anOptional: Boolean);
    Begin
      Inherited Create( aCondition, anOptional);
      FValue := aValue;
      FSize  := aSize;
    End;

    Function    TConstantMatcher.Match(
      aProtocol : TProtocol;
      aData     : TBitStream;
      aPacket   : TPacket
    ): Boolean; // Override;
    Begin
      Trace( aProtocol);
      Result := aData.HasAvailable( FSize) And ( aData.GetInt( FSize) = FValue);
    End;

    Function    TConstantMatcher.Apply(
      aProtocol  : TProtocol;
      aPacket    : TPacket;
      aData      : TIntStream;
      aBitStream : TBitStream
    ): Boolean; // Override;
    Begin
      Trace( aProtocol);
      aBitStream.Append( FValue, FSize);
      Result := True;
    End;


{ ========
  TVariableMatcher = Class( TMatcher)
  Private
    FVariable : String;
    FCount    : Integer;
    FSize     : Integer;
    FTerminal : Integer;
  Private
}

    Procedure   TVariableMatcher.Trace( aProtocol: TProtocol);
    Begin
      If FCount = 1
      Then aProtocol.Trace( Copy( Format( '  %s:%d'      , [ FVariable, FSize                   ]), 1, 100))
      Else aProtocol.Trace( Copy( Format( '  %d*%s:%d/%d', [ FCount, FVariable, FSize, FTerminal]), 1, 100));
    End;

//   Public

    Constructor TVariableMatcher.Create(
      aCount          : Integer;
      Const aVariable : String;
      aSize           : Integer;
      aTerminal       : Integer;
      aCondition      : TCondition;
      anOptional      : Boolean
    );
    Begin
      Inherited Create( aCondition, anOptional);
      FCount    := aCount;
      FVariable := aVariable;
      FSize     := aSize;
      FTerminal := aTerminal;
    End;


    Function    TVariableMatcher.Match(
      aProtocol : TProtocol;
      aData     : TBitStream;
      aResult   : TPacket
    ): Boolean; // Override;
    Var
      aVariableList : TVariableList;
      i             : Integer;
      aValue        : Integer;
    Begin
      aVariableList := TVariableList.Create;

      Trace( aProtocol);

      aValue := -1;
      For i := 0 To FCount - 1 Do
      Begin
        If aData.HasAvailable( FSize)
        Then Begin
          aValue := aData.GetInt( FSize);
          aVariableList.Push( aValue);
          If avalue = FTerminal
          Then Break;
        End
        Else Begin
          aVariableList.Free;
          Result := False;
          Exit;
        End;
      End;

      If FCount = 1
      Then Begin
        aResult.Bind( aValue, FVariable);
        With aVariableList Do
        Begin
          Clear;
          Free;
        End;
      End
      Else aResult.Bind( aVariableList, FVariable);
      Result := True;
    End;

    Function    TVariableMatcher.Apply(
      aProtocol : TProtocol;
      aPacket   : TPacket;
      aData     : TIntStream;
      aResult   : TBitStream
    ): Boolean; // Override;
    Var
      aVariableList : TVariableList;
      i             : Integer;
      aValue        : Integer;
    Begin
      aVariableList := TVariableList.Create;

      Trace( aProtocol);

      aValue := -1;
      For i := 0 To FCount - 1 Do
      Begin
        If aData.HasAvailable( 1)
        Then Begin
          aValue := aData.GetInt;
          aVariableList.Push( aValue);
          aResult.Append( aValue, FSize);
          If aValue = FTerminal
          then Break;
        End
        Else Begin
          Result := False;
          aVariableList.Free;
          Exit;
        End;
      End;

      If FCount = 1
      Then Begin
        aPacket.Bind( aValue, FVariable);
        aVariableList.Free;
      End
      Else aPacket.Bind( aVariableList, FVariable);
      Result := true;
    End;


{ ========
  TPacketMatcher = Class( TMatcher)
  Private
    FCount      : String;
    FParserName : String;
    FBinding    : String;
  Private
}

    Procedure   TPacketMatcher.Trace( aProtocol: TProtocol);
    Begin
      If FCount = ''
      Then aProtocol.TraceFmt( '  %s$%s'   , [         FParserName, FBinding])
      Else aProtocol.TraceFmt( '  %s*%s$%s', [ FCount, FParserName, FBinding]);
    End;

//  Public

    Constructor TPacketMatcher.Create(
      Const aCount      : String;
      Const aParserName : String;
      Const aBinding    : String;
      aCondition        : TCondition;
      anOptional        : Boolean
    );
    Begin
      Inherited Create( aCondition, anOptional);
      FCount      := aCount;
      FParserName := aParserName;
      FBinding    := aBinding;
    End;

    Function    TPacketMatcher.Match(
      aProtocol : TProtocol;
      aData     : TBitStream;
      aResult   : TPacket
    ): Boolean; // Override;
    Var
      Iterations   : Integer;
      PacketList   : TPacketList;
      PacketParser : TPacketParser;
      Packet       : Tpacket;
      i            : Integer;
    Begin
      PacketParser := aProtocol.PacketParser[ FParserName];
      If Not Assigned( PacketParser)
      Then Begin
        Result := False;
        Exit;
      End;

      Trace( aProtocol);

      If FCount = ''
      Then Iterations := 1
      Else Begin
        Iterations := aResult.Variable[ FCount];
        If Iterations < 0
        Then Begin
          Result := False;
          Exit;
        End;
      End;

      PacketList := TPacketList.Create;
      For i := 0 To Iterations - 1 Do
      Begin
        Packet := TPacket.Create;
        If Packetparser.Parse( aData, Packet)
        Then PacketList.Push( Packet)
        Else Begin
          FreeAndNil( Packet);
          FreeAndNil( PacketList);
          Result := False;
          Exit;
        End;
      End;

      If FCount = ''
      Then Begin
        aResult.Bind( Packet, FBinding);
        PacketList.Clear;
        FreeAndNil( PacketList);
      End
      Else aResult.Bind( PacketList, FBinding);
      Result := True;
    End;

    Function    TPacketMatcher.Apply(
      aProtocol : TProtocol;
      aPacket   : TPacket;
      aData     : TIntStream;
      aResult   : TBitStream
    ): Boolean; // Override;
    Var
      Iterations   : Integer;
      PacketParser : TPacketParser;
      i            : Integer;
    Begin
      PacketParser := aProtocol.PacketParser[ FParserName];
      If Not Assigned( PacketParser)
      Then Begin
        Result := False;
        Exit;
      End;

      Trace( aProtocol);

      If FCount = ''
      Then Iterations := 1
      Else Begin
        Iterations := aPacket.Variable[ FCount];
        If Iterations < 0
        Then Begin
          Result := False;
          Exit;
        End;
      End;

      For i := 0 To Iterations - 1 Do
      Begin
        If Not PacketParser.Generate( aData, aResult)
        Then Begin
          Result := False;
          Exit;
        End;
      End;
      Result := True;
    End;

    
{ ========
  TPacketParser = Class
  Private
    FName     : String;
    FPadding  : Integer;
    FProtocol : TProtocol;
    FMatchers : TMatcherList;
  Public
}

    Constructor TPacketParser.Create( Const aName: String; aPadding: Integer; aProtocol: TProtocol);
    Begin
      Inherited Create;
      FName     := aName;
      FPadding  := aPadding;
      FProtocol := aProtocol;
      FMatchers := TMatcherList.Create;
    End;

    Destructor  TPacketParser.Destroy; // Override;
    Begin
      FreeAndNil( FMatchers);
      Inherited;
    End;

    Procedure   TPacketParser.AddPacketMatcher(
      Const aCount      : String;
      Const aPacketName : String;
      Const aBinding    : String;
      aCondition        : TCondition;
      anOptional        : Boolean
    );
    Begin
      FMatchers.
        Push(
          TPacketMatcher.
            Create(
              aCount,
              aPacketName,
              aBinding,
              aCondition,
              anOptional
            )
        );
    End;

    Procedure   TPacketParser.AddVariableMatcher(
      aCount     : Integer;
      aVariable  : String;
      aSize      : Integer;
      aTerminal  : Integer;
      aCondition : TCondition;
      anOptional : Boolean
    );
    Begin
      FMatchers.
        Push(
          TVariableMatcher.
            Create(
              aCount,
              aVariable,
              aSize,
              aTerminal,
              aCondition,
              anOptional
            )
        );
    End;

    Procedure   TPacketParser.AddConstantMatcher(
      aConstant  : Integer;
      aSize      : Integer;
      aCondition : TCondition;
      anOptional : Boolean
    );
    Begin
      FMatchers.
        Push(
          TConstantMatcher.
            Create(
              aConstant,
              aSize,
              aCondition,
              anOptional
            )
        );
    End;

    Function    TPacketParser.Parse( aData: TBitStream; aResult: TPacket): Boolean;
    Var
      Conditional      : Boolean;
      ConditionalMatch : Boolean;
      DataPos          : Integer;
      i                : Integer;
      Tmp              : Integer;
    Label
      parse_failed;
    Begin
      Result           := False;
      Conditional      := False;
      ConditionalMatch := False;
      DataPos          := aData.Position;

      FProtocol.TraceFmt( '** PARSE PACKET %s', [ FName]);

      aResult.Name := FName;

      With FMatchers Do
      Begin
        For i := 0 To Size - 1 Do
        Begin
          If Items[ i].IsConditional
          Then Begin
            Conditional := True;
            If Items[ i].TrueCondition( aResult)
            Then Begin
              If Items[ i].Match( FProtocol, aData, aResult)
              Then Begin
                ConditionalMatch := True;
                // Break;
              End;
            End
          End
          Else Begin
            If Not Items[ i].Match( FProtocol, aData, aResult) And Not Items[ i].IsOptional
            Then Goto parse_failed;
          End;
        End;
      End;

      If Not Conditional Or ( Conditional And ConditionalMatch)
      Then Begin
        Tmp := ( aData.Position - DataPos) Mod FPadding;
        If Tmp = 0
        Then aData.GetInt( 0)
        Else aData.GetInt( FPadding - Tmp);
        FProtocol.TraceFmt( '** PARSE MATCHED %s', [ FName]);
        Result := True;
      End;

      Exit;

    parse_failed:

      FProtocol.TraceFmt( '** PARSE FAILED %s', [ FName]);
      aData.Position := DataPos;
      aResult.Clear;
      Result := False;
    End;

    Function    TPacketParser.Generate( aData: TIntStream; aResult: TBitStream): Boolean;
    Var
      Packet           : TPacket;
      Conditional      : Boolean;
      ConditionalMatch : Boolean;
      DataPos          : Integer;
      ResultSize       : Integer;
      i                : Integer;
      Success          : Boolean;
      Tmp              : Integer;
    Label
      generate_failed;
    Begin
      Packet           := TPacket.Create;
      Conditional      := False;
      ConditionalMatch := False;
      DataPos          := aData.Position;
      ResultSize       := aResult.Size;

      FProtocol.TraceFmt( '** GENERATE PACKET %s', [ FName]);

      With FMatchers Do
      Begin
        For i := 0 To Size - 1 Do
        Begin
          If Items[ i].IsConditional
          Then Begin
            Conditional := True;
            If Items[ i].TrueCondition( Packet)
            Then Begin
              Success := Items[ i].Apply( FProtocol, Packet, aData, aResult);
              If Success
              Then ConditionalMatch := True;
            End
          End
          Else Begin
            Success := Items[ i].Apply( FProtocol, Packet, aData, aResult);
            If Not Success And Not Items[ i].IsOptional
            Then Begin
              Noop;
              Goto generate_failed;
            End;
          End;
        End;
      End;

      If Not Conditional Or ( Conditional And ConditionalMatch)
      Then Begin
        Tmp := ( aResult.Size - ResultSize) Mod FPadding;
        If Tmp = 0
        Then aResult.Append( 0, Tmp)
        Else aResult.Append( 0, FPadding - Tmp);
      End;

      FProtocol.TraceFmt( '** GENERATE MATCHED %s', [ FName]);
      Result := True;

      Exit;

    generate_failed:

      FProtocol.TraceFmt( '** GENERATE FAILED %s', [ FName]);
      aData.Position := DataPos;
      aResult.Size := ResultSize;
      Packet.Free;
      Result := False;
    End;


{ ========
  TProtocol = Class
  Private
    FTracer        : TTracer;
    FPacketParsers : TPacketparserMap;
  Public
    Property    PacketParser[ Const anIndex: String]: TPacketparser Read GetPacketparser;
  Private
}

    Function    TProtocol.GetPacketParser( Const anIndex: String): TPacketParser;
    Begin
      Result := FPacketParsers[ anIndex];
    End;

    Procedure   TProtocol.DoParseError( aSender: TObject; Const aMsg: String; Var AbortParsing: Boolean);
    Begin
      Trace( aMsg);
      AbortParsing := False; // Let parser continue.
    End;

// Public

    Constructor TProtocol.Create;
    Begin
      Inherited Create;
      FPacketParsers := TPacketParserMap.Create;
    End;

    Destructor  TProtocol.Destroy; // Override;
    Begin
      FreeAndNil( FPacketParsers);
      UseTracer( Nil);
      Inherited;
    End;

    Procedure   TProtocol.LoadPdlFile( Const aFileName: String; DoYaccDebug: Boolean = False);
    Var
      Parser : TParser; // parser as generated by Lex and Yacc in pdlparse.pas
      i      : Integer;
    Begin
      Parser  := TParser.Create;                       // Create the pdl parser.
      With Parser Do
      Try
        Protocol     := Self;
        OnParseError := DoParseError;                  // Install an error handler for it.
        Debug        := DoYaccDebug;                   // Pass on debug preference.
        PushFile( aFileName);                          // Make it parse aFileName
        Try
          If ( Parse <> 0) Or ( ErrorCount <> 0)       // Parse it and if any untrapped errors
          Then Abort;                                  //   occurred, trap them now.
        Except
          On Exception Do
            Raise PdlException.Create( 'Could not parse pdl file', 0);
        End;
      Finally
        With Messages Do                               // Collect debug messages ...
          For i := 0 To Count - 1 Do
            Trace( Strings[ i]);                       //   ... and show them
        Free;                                          // Destroy the pdl parser.
      End;
    End;

    Procedure   TProtocol.UseTracer( aTracer: TTracer);
    Begin
      If Assigned( FTracer)
      Then FreeAndNil( FTracer);
      FTracer := aTracer;
    End;

    Procedure   TProtocol.Trace( Const aMsg: String);
    Begin
      If Assigned( FTracer)
      Then FTracer.Trace( aMsg);
      // WriteDebugStr( aMsg);
    End;

    Procedure   TProtocol.TraceFmt( Const aFormat: String; Const anArgs: Array Of Const);
    Begin
      Trace( Format( aFormat, anArgs));
    End;

    Function    TProtocol.CreatePacketParser( Const aName: String; aPadding: Integer): TPacketParser;
    Begin
      Result := TPacketParser.Create( aName, aPadding, Self);
      FPacketParsers[ aName] := Result;
    End;



End.

